@@ -7,6 +7,7 @@
 //
 // 07/06/2007
 // 12/20/2007 included in MCMCpack
+// 03/09/2012 fixed a bug in a random uniform draw (thanks to Matt Blackwell)
 
 #ifndef MCMCPOISSONCHANGE_CC
 #define MCMCPOISSONCHANGE_CC
@@ -85,7 +86,12 @@ Matrix<> tau_comp_sampler(rng<RNGTYPE>& stream,
     }
     
     else {
-      Matrix<> ut = stream.runif(yt, 1);                       
+      Matrix<> ut = stream.runif(yt, 1);    
+      // redraw the uniform if there are any repeats
+      // thanks to Matt Blackwell 
+      while (::unique(ut).size() != ut.size()) {
+	ut = stream.runif(yt, 1);
+      }                   
       Matrix<> sort_ut = ::sort(ut);
       Matrix<> tau_tj(yt, 1);
       for(int i=1; i<yt; ++i){
@@ -298,6 +304,7 @@ void MCMCpoissonChangepoint_impl(rng<RNGTYPE>& stream,
 				 const double *b, 
 				 const double *A0data, 
 				 double *logmarglikeholder, 
+				 double *loglikeholder, 
 				 const int *chib)
 
 {
@@ -506,6 +513,7 @@ void MCMCpoissonChangepoint_impl(rng<RNGTYPE>& stream,
     const double logmarglike = (loglike + logprior) - (pdf_lambda + pdf_P); 
     
     logmarglikeholder[0] = logmarglike;
+    loglikeholder[0] = loglike;
   }
   
   R_CheckUserInterrupt();      
@@ -556,6 +564,7 @@ void MCMCpoissonRegChangepoint_impl(rng<RNGTYPE>& stream,
 				    const double *B0data, 
 				    const double *A0data, 
 				    double *logmarglikeholder, 
+				    double *loglikeholder, 
 				    const double* wrin, 
 				    const double* mrin, 
 				    const double* srin, 
@@ -643,8 +652,8 @@ void MCMCpoissonRegChangepoint_impl(rng<RNGTYPE>& stream,
       for (int h = 0; h<nstate[j]; ++h){            
 	Xwj(h, _) = Xj(h,_)*wi[h];
       }
-      Matrix<> Bn = invpd(B0inv + ::t(Xj)*Xwj);
-      Matrix<> bn = Bn*gaxpy(B0inv, b0,  -1*::t(Xj)*yj);
+      Matrix<> Bn = invpd(B0 + ::t(Xj)*Xwj);
+      Matrix<> bn = Bn*gaxpy(B0, b0,  -1*::t(Xj)*yj);
       beta(j,_) = stream.rmvnorm(bn, Bn);
    }
     
@@ -757,8 +766,8 @@ void MCMCpoissonRegChangepoint_impl(rng<RNGTYPE>& stream,
 	for (int h = 0; h<nstate[j]; ++h){            
 	  Xwj(h, _) = Xj(h,_)*wi[h];
 	}
-	  Matrix<> Bn = invpd(B0inv + ::t(Xj)*Xwj);
-	  Matrix<> bn = Bn*gaxpy(B0inv, b0,  -1*::t(Xj)*yj);         
+	  Matrix<> Bn = invpd(B0 + ::t(Xj)*Xwj);
+	  Matrix<> bn = Bn*gaxpy(B0, b0,  -1*::t(Xj)*yj);         
 	  density_beta(iter, j) = exp(lndmvn(::t(beta_st(j,_)), bn, Bn));	  
       }       
     }   
@@ -838,7 +847,7 @@ void MCMCpoissonRegChangepoint_impl(rng<RNGTYPE>& stream,
     density_P[ns-1] = 1; //
     
     for (int j=0; j<ns ; ++j){
-      density_beta_prior[j] = lndmvn(::t(beta_st(j,_)), b0, B0);    
+      density_beta_prior[j] = lndmvn(::t(beta_st(j,_)), b0, B0inv);    
     }   
     
     for (int j =0; j< (ns-1); ++j){
@@ -848,9 +857,16 @@ void MCMCpoissonRegChangepoint_impl(rng<RNGTYPE>& stream,
     // compute marginal likelihood
     const double logprior = sum(density_beta_prior) + sum(density_P_prior);
     const double logmarglike = (loglike + logprior) - (pdf_beta + pdf_P);
-
+    if (verbose >0 ){
+      Rprintf("\nlogmarglike = %10.5f\n", logmarglike);
+      Rprintf("loglike = %10.5f\n", loglike);
+      Rprintf("log_prior = %10.5f\n", logprior);
+      Rprintf("log_beta = %10.5f\n", pdf_beta);
+      Rprintf("log_P = %10.5f\n", pdf_P);
+    }
     logmarglikeholder[0] = logmarglike;
-    
+    loglikeholder[0] = loglike;
+   
   }
         
   R_CheckUserInterrupt();      
@@ -901,6 +917,7 @@ extern "C" {
 			  const double *B0data, 
 			  const double *A0data, 
 			  double *logmarglikeholder, 
+			  double *loglikeholder, 
 			  const double *wrin, 
 			  const double *mrin, 
 			  const double *srin, 
@@ -913,7 +930,7 @@ extern "C" {
 			     burnin, mcmc, thin, verbose,  
 			     betastart, Pstart, 
 			     a, b, A0data,
-			     logmarglikeholder, chib)
+			     logmarglikeholder, loglikeholder, chib)
 	}
     else{
       MCMCPACK_PASSRNG2MODEL(MCMCpoissonRegChangepoint_impl, 
@@ -924,7 +941,7 @@ extern "C" {
 			     betastart, Pstart, 
 			     taustart, componentstart,
 			     a, b, b0data, B0data, A0data, 
-			     logmarglikeholder, 
+			     logmarglikeholder, loglikeholder, 
 			     wrin, mrin, srin, chib);
     }
   } // end MCMC
